
package Drivers.startrek;


import Allegory.startrek.Database;
import Allegory.startrek.Lang.DTable;
import Allegory.startrek.crewmanTable;
import Allegory.startrek.medical;
import Allegory.startrek.postTable;
import PrologDB.ErrorReport;

public class Driver {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Database st = new Database("test/Allegory/startrek/TestData/cast.startrek.pl");
        ErrorReport er = new ErrorReport();
        
        // queries
        // Q1: list the name, posting of each vulcan
        System.out.println("Q1: list the (name, post) for each vulcan");
///Q1
        DTable Q1 = new DTable("Q1", "vname", "vposting");
        st.crewman.select(c->c.species.equals("vulcan"))
                .forEach(v ->v.onBoard().forEach(d->Q1.add(v.fname,d.name)));
        Q1.print();
///Q1
        
        // Q2: which medical officers worked on deep station 9
        System.out.println("\n\nQ2: which medical officers worked on deep station 9?");
///Q2
        st.post.select(p->p.name.equals("Deep Space 9")).hasCrew()
                .select(c->(c instanceof Allegory.startrek.medical)).print();
        System.out.println("version 2");
        st.medical.select(m->m.onBoard()
                .select(mm->mm.name.equals("Deep Space 9")).exists()).print();
///Q2

        // Q3: who worked on Enterprise-A?
        System.out.println("\n\nQ3: who worked on Enterprise-A?");
///Q3
        st.post.select(p->p.name.equals("Enterprise-A")).hasCrew().print();
///Q3
        
        // Q4: what ships were riker and troi both assigned?
        System.out.println("Q4: on what ships were riker and troi both assigned?");
///Q4
        postTable dt = st.post.select(p->p.hasCrew().select(c->c.lname.equals("troi")).exists());
        postTable wr = st.post.select(p->p.hasCrew().select(c->c.lname.equals("riker")).exists());
        postTable both = dt.intersect(wr);
        both.print();
///Q4
        
        // Q5: chief medical officers are on what posts?
        System.out.println("Q5: chief medical officers are on what posts?");
///Q5
        st.medical.select(m->m.specialty.equals("chief")).onBoard().print();
        System.out.println("version 2");
        st.post.select(
                p->p.hasCrew()
                .select(c->((c instanceof medical) && ((medical) c).specialty.equals("chief"))).exists())
                .print();
///Q5
        
        // Q6: example of field projection
        System.out.println("Q6: what are first names of officers on Enterprise-A");
///Q6
        st.post.select(p->p.name.equals("Enterprise-A")).hasCrew().fname().print();
///Q6
        
        
        // constraints
        // C1: vulcans can only work abord star ships
///C1
        st.crewman.select(c->c.species.equals("vulcan"))
                .forEach(v->v.onBoard()
                        .reject(p->p.kind.equals("star ship"))
                        .error(er,"C1: %v is not onboard a starship", e->e.name));
///C1
        
        // C2: if a klingon works on ds9, then so too must a trill
///C2
        crewmanTable onds9 = st.post.select(p->p.name.equals("Deep Space 9")).hasCrew();
        if (onds9.exists(p->p.species.equals("klingon"))) {
            if (onds9.notExist(p->p.species.equals("trill")))
                er.add("C2: a klingon works on ds9, but no trills");
        }
///C2
        

        
        // report any errors
        System.out.println("error report:");
        er.print(System.out);
               
        
    }
    
}
